var through = require("through");
var format = require("format-text");
var filterStack = require("filter-stack");
var prettifyError = require("prettify-error");
var template = require("./node-template");

var failTemplate = template('fail');
var resultPassTemplate = template('result-pass');
var resultTemplate = template('result');
var diffTemplate = template('diff');
var command = require("./node-command");

var progress = require('./progress');
var tests = require("./tests");

var failed = 0;
var passed = 0;
var excludeFromStack = ['prova'];

module.exports = reporter;
module.exports.diff = diff;
module.exports.tab = tab;

function reporter () {
  return through(each, end);
}

function each (row) {
  if (row.type == 'assert' && row.ok) {
    passed++;
  } else if (row.type == 'assert') {
    failed++;
    fail(row);
  }

  if (command.progress && row.type == 'test') {
    tests.done(row.name);
    progress.show(row);
  }
}

function fail (row) {
  var error = row.actual && row.actual.stack ? row.actual : row.error;
  error.stack = filterStack(error, excludeFromStack).stack;

  var startingLine = 0;
  var stackLines = error.stack.split('\n');
  var len = stackLines.length;
  var found = false;
  while (++startingLine < len) {
    if (/node_modules\/tape\//.test(stackLines[startingLine])) continue;
    found = true;
    break;
  }

  if (!found) startingLine = 1;

  var prettifiedError = prettifyError(error, startingLine - 1);

  if (!prettifiedError) {
    prettifiedError = '\n  ' + error.message + '\n' + tab(error.stack, '  ');
  }

  console.error(format(failTemplate, {
    title: row.testName,
    diff: diff(row),
    error: tab(prettifiedError, '    ')
  }));
}

function diff (row) {
  if (!row.hasOwnProperty('expected')) return '';
  return format(diffTemplate, JSON.stringify(row.expected), JSON.stringify(row.actual)) + '\n';
}

function end () {
  (command.progress ? progress.end : console.log)(format(failed ? resultTemplate : resultPassTemplate, passed, failed));

  this.queue(null);

  if (command.quit) {
    process.exit(failed);
  }
}

function tab (text, ch) {
  return ch + text.replace(/\n/g, '\n' + ch);
}
